@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-embed-single.c,v 1.34 2005/02/14 20:57:07 chpe Exp $
+ *  $Id: ephy-embed-single.c,v 1.36 2005/07/24 21:37:01 chpe Exp $
  */
 
 #include "config.h"
@@ -137,8 +137,8 @@ ephy_embed_single_iface_init (gpointer g_class)
 		      ephy_marshal_BOOLEAN__STRING_STRING,
 		      G_TYPE_BOOLEAN,
 		      2,
-		      G_TYPE_STRING,
-		      G_TYPE_STRING);
+		      G_TYPE_STRING | G_SIGNAL_TYPE_STATIC_SCOPE,
+		      G_TYPE_STRING | G_SIGNAL_TYPE_STATIC_SCOPE);
 
 /**
  * EphyEmbedSingle::add-search-engine
@@ -158,9 +158,9 @@ ephy_embed_single_iface_init (gpointer g_class)
 		      ephy_marshal_BOOLEAN__STRING_STRING_STRING,
 		      G_TYPE_BOOLEAN,
 		      3,
-		      G_TYPE_STRING,
-		      G_TYPE_STRING,
-		      G_TYPE_STRING);
+		      G_TYPE_STRING | G_SIGNAL_TYPE_STATIC_SCOPE,
+		      G_TYPE_STRING | G_SIGNAL_TYPE_STATIC_SCOPE,
+		      G_TYPE_STRING | G_SIGNAL_TYPE_STATIC_SCOPE);
 
 /**
  * EphyEmbedSingle::check_content:
@@ -304,10 +304,3 @@ ephy_embed_single_open_window (EphyEmbedSingle *single,
 	EphyEmbedSingleIface *iface = EPHY_EMBED_SINGLE_GET_IFACE (single);
 	return iface->open_window (single, parent, address, name, features);
 }
-
-GList *
-ephy_embed_single_get_printer_list (EphyEmbedSingle *single)
-{
-	EphyEmbedSingleIface *iface = EPHY_EMBED_SINGLE_GET_IFACE (single);
-	return iface->get_printer_list (single);
-}
