@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-history.c,v 1.58 2005/04/08 17:02:52 chpe Exp $
+ *  $Id: ephy-history.c,v 1.60 2005/07/31 20:35:26 jframeau Exp $
  */
 
 #include "config.h"
@@ -34,8 +34,8 @@
 #include <glib/gi18n.h>
 #include <libgnomevfs/gnome-vfs-uri.h>
 
-#define EPHY_HISTORY_XML_ROOT	 "ephy_history"
-#define EPHY_HISTORY_XML_VERSION "1.0"
+#define EPHY_HISTORY_XML_ROOT	 (const xmlChar *)"ephy_history"
+#define EPHY_HISTORY_XML_VERSION (const xmlChar *)"1.0"
 
 /* how often to save the history, in milliseconds */
 #define HISTORY_SAVE_INTERVAL (5 * 60 * 1000)
@@ -54,11 +54,11 @@ struct _EphyHistoryPrivate
 	EphyNode *last_page;
 	GHashTable *hosts_hash;
 	GHashTable *pages_hash;
-	int autosave_timeout;
+	guint autosave_timeout;
 	guint update_hosts_idle;
+	guint disable_history_notifier_id;
 	gboolean dirty;
 	gboolean enabled;
-	guint disable_history_notifier_id;
 };
 
 enum
@@ -277,7 +277,7 @@ ephy_history_save (EphyHistory *eb)
 	LOG ("Saving history db");
 
 	ret = ephy_node_db_write_to_xml_safe
-		(eb->priv->db, eb->priv->xml_file,
+		(eb->priv->db, (const xmlChar *)eb->priv->xml_file,
 		 EPHY_HISTORY_XML_ROOT,
 		 EPHY_HISTORY_XML_VERSION,
 		 NULL, /* comment */
