@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: EphyBrowser.h,v 1.45 2005/07/12 20:47:21 chpe Exp $
+ *  $Id: EphyBrowser.h,v 1.46 2005/07/28 17:52:50 chpe Exp $
  */
 
 #ifndef EPHY_BROWSER_H
@@ -34,6 +34,7 @@
 #include <nsIWebNavigation.h>
 #include <nsISHistory.h>
 #include <nsIWebBrowser.h>
+#include <nsIWebBrowserFocus.h>
 #include <nsIDOMDocument.h>
 #include <nsIDOMWindow.h>
 #include <nsIPrintSettings.h>
@@ -178,10 +179,16 @@ public:
 
         EphyEmbedDocumentType GetDocumentType ();
 
+#ifdef GTKMOZEMBED_BROKEN_FOCUS
+	nsresult FocusActivate ();
+	nsresult FocusDeactivate ();
+#endif
+
 	nsCOMPtr<nsIWebBrowser> mWebBrowser;
 private:
 	GtkWidget *mEmbed;
 
+	nsCOMPtr<nsIWebBrowserFocus> mWebBrowserFocus;
 	nsCOMPtr<nsIDOMDocument> mTargetDocument;
 	nsCOMPtr<nsIDOMEventTarget> mEventTarget;
 	nsCOMPtr<nsIDOMWindow> mDOMWindow;
