@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: GlobalHistory.h,v 1.9 2004/12/02 15:16:41 chpe Exp $
+ *  $Id: GlobalHistory.h,v 1.10 2005/01/01 20:54:18 chpe Exp $
  */
 
 #ifndef EPHY_GLOBAL_HISTORY_H
@@ -24,7 +24,6 @@
 
 #include "ephy-history.h"
 
-#include <nsIBrowserHistory.h>
 #include <nsIGlobalHistory2.h>
 
 #include <nsCOMPtr.h>
@@ -41,7 +40,7 @@
 	{ 0xb7, 0x9e, 0xf7, 0xaa, 0x49, 0xeb, 0x6a, 0x15}	\
 }
 
-class MozGlobalHistory: public nsIBrowserHistory
+class MozGlobalHistory: public nsIGlobalHistory2
 {
 	public:
 		MozGlobalHistory ();
@@ -49,7 +48,6 @@ class MozGlobalHistory: public nsIBrowserHistory
 
 		NS_DECL_ISUPPORTS
 		NS_DECL_NSIGLOBALHISTORY2
-		NS_DECL_NSIBROWSERHISTORY
 
 	private:
 		EphyHistory *mGlobalHistory;
