@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: mozilla-embed.cpp,v 1.115 2005/08/06 22:26:45 chpe Exp $
+ *  $Id: mozilla-embed.cpp,v 1.117 2005/08/13 14:33:26 chpe Exp $
  */
 
 #include "mozilla-config.h"
@@ -226,7 +226,6 @@ static void
 mozilla_embed_realize (GtkWidget *widget)
 {
 	MozillaEmbedPrivate *mpriv = MOZILLA_EMBED (widget)->priv;
-	GtkBin *bin = GTK_BIN (widget);
 
 	GTK_WIDGET_CLASS (parent_class)->realize (widget);
 
@@ -269,7 +268,7 @@ mozilla_embed_realize (GtkWidget *widget)
 
 	if (mpriv->focus_connected) return;
 
-	GtkWidget *child = gtk_bin_get_child (bin);
+	GtkWidget *child = gtk_bin_get_child (GTK_BIN (widget));
 	g_return_if_fail (child != NULL);
 
 	n = g_signal_handlers_block_matched (child,
@@ -338,25 +337,25 @@ mozilla_embed_init (MozillaEmbed *embed)
         embed->priv = MOZILLA_EMBED_GET_PRIVATE (embed);
 	embed->priv->browser = new EphyBrowser ();
 
-	g_signal_connect_object (G_OBJECT (embed), "location",
+	g_signal_connect_object (embed, "location",
 				 G_CALLBACK (mozilla_embed_location_changed_cb),
 				 embed, (GConnectFlags) 0);
-	g_signal_connect_object (G_OBJECT (embed), "net_state_all",
+	g_signal_connect_object (embed, "net_state_all",
 				 G_CALLBACK (mozilla_embed_net_state_all_cb),
 				 embed, (GConnectFlags) 0);
-	g_signal_connect_object (G_OBJECT (embed), "dom_mouse_click",
+	g_signal_connect_object (embed, "dom_mouse_click",
 				 G_CALLBACK (mozilla_embed_dom_mouse_click_cb),
 				 embed, (GConnectFlags) 0);
-	g_signal_connect_object (G_OBJECT (embed), "dom_mouse_down",
+	g_signal_connect_object (embed, "dom_mouse_down",
 				 G_CALLBACK (mozilla_embed_dom_mouse_down_cb),
 				 embed, (GConnectFlags) 0);
-	g_signal_connect_object (G_OBJECT (embed), "dom-key-press",
+	g_signal_connect_object (embed, "dom-key-press",
 				 G_CALLBACK (mozilla_embed_dom_key_press_cb),
 				 embed, (GConnectFlags) 0);
-	g_signal_connect_object (G_OBJECT (embed), "new_window",
+	g_signal_connect_object (embed, "new_window",
 				 G_CALLBACK (mozilla_embed_new_window_cb),
 				 embed, (GConnectFlags) 0);
-	g_signal_connect_object (G_OBJECT (embed), "security_change",
+	g_signal_connect_object (embed, "security_change",
 				 G_CALLBACK (mozilla_embed_security_change_cb),
 				 embed, (GConnectFlags) 0);
 }
