@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- *  $Id: ephy-find-toolbar.c,v 1.12 2005/07/29 20:40:38 crispin Exp $
+ *  $Id: ephy-find-toolbar.c,v 1.16 2005/08/15 22:52:51 chpe Exp $
  */
 
 #include "config.h"
@@ -40,6 +40,8 @@
 #include <gtk/gtkmain.h>
 #include <string.h>
 
+#undef FIND_WHILE_TYPING_IN_EMBED
+
 #define EPHY_FIND_TOOLBAR_GET_PRIVATE(object) (G_TYPE_INSTANCE_GET_PRIVATE ((object),EPHY_TYPE_FIND_TOOLBAR, EphyFindToolbarPrivate))
 
 struct _EphyFindToolbarPrivate
@@ -56,6 +58,7 @@ struct _EphyFindToolbarPrivate
 	guint activated : 1;
 	guint explicit_focus : 1;
 	guint links_only : 1;
+	guint typing_ahead : 1;
 };
 
 enum
@@ -118,6 +121,8 @@ tab_content_changed_cb (EphyEmbed *embed,
 
 #ifdef HAVE_TYPEAHEADFIND
 
+#ifdef FIND_WHILE_TYPING_IN_EMBED
+
 /* Cut and paste from gtkwindow.c */
 static void
 send_focus_change (GtkWidget *widget,
@@ -150,6 +155,8 @@ send_focus_change (GtkWidget *widget,
 	gdk_event_free (event);
 }
 
+#endif /* FIND_WHILE_TYPING_IN_EMBED */
+
 /* Code adapted from gtktreeview.c:gtk_tree_view_key_press() and
  * gtk_tree_view_real_start_interactive_seach()
  */
@@ -160,10 +167,12 @@ tab_search_key_press_cb (EphyEmbed *embed,
 {
 	EphyFindToolbarPrivate *priv = toolbar->priv;
 	GtkWidget *widget = (GtkWidget *) toolbar;
+#ifdef FIND_WHILE_TYPING_IN_EMBED
 	GtkEntry *entry = (GtkEntry *) priv->entry;
 	GdkWindow *event_window;
 	gboolean retval = FALSE;
 	guint oldhash, newhash;
+#endif
 
 	g_return_val_if_fail (event != NULL, FALSE);
 
@@ -184,7 +193,10 @@ tab_search_key_press_cb (EphyEmbed *embed,
 			return TRUE;
 		}
 	}
-	
+
+	return FALSE;
+
+#ifdef FIND_WHILE_TYPING_IN_EMBED
 	/* don't do anything if the find toolbar is hidden */
 	if (GTK_WIDGET_VISIBLE (widget) == FALSE ||
 	    event->keyval == GDK_Return ||
@@ -212,8 +224,11 @@ tab_search_key_press_cb (EphyEmbed *embed,
 
 	/* FIXME: is this correct? */
 	return retval && (oldhash != newhash || priv->preedit_changed) && !priv->activated;
+#endif /* FIND_WHILE_TYPING_IN_EMBED */
 }
 
+#ifdef FIND_WHILE_TYPING_IN_EMBED
+
 static gboolean
 tab_dom_mouse_click_cb (EphyEmbed *embed,
 			gpointer event,
@@ -229,6 +244,8 @@ tab_dom_mouse_click_cb (EphyEmbed *embed,
 	return FALSE;
 }
 
+#endif /* FIND_WHILE_TYPING_IN_EMBED */
+
 #endif /* HAVE_TYPEAHEADFIND */
 
 static void
@@ -268,12 +285,14 @@ entry_changed_cb (GtkEntry *entry,
 	set_controls (toolbar, found, found);
 }
 
+#ifdef FIND_WHILE_TYPING_IN_EMBED
 static void
 entry_preedit_changed_cb (GtkIMContext *context,
 			  EphyFindToolbar *toolbar)
 {
 	toolbar->priv->preedit_changed = TRUE;
 }
+#endif
 
 static gboolean
 entry_key_press_event_cb (GtkEntry *entry,
@@ -289,6 +308,11 @@ entry_key_press_event_cb (GtkEntry *entry,
 		g_signal_emit (toolbar, signals[CLOSE], 0);
 		handled = TRUE;
 	}
+	else if ((event->state & mask) == GDK_CONTROL_MASK &&
+		 (event->keyval == GDK_Return || event->keyval == GDK_KP_Enter))
+	{
+		handled = ephy_embed_find_activate_link (get_find (toolbar), event->state);
+	}
 
 	return handled;
 }
@@ -299,10 +323,19 @@ entry_activate_cb (GtkWidget *entry,
 {
 	EphyFindToolbarPrivate *priv = toolbar->priv;
 
+#ifdef FIND_WHILE_TYPING_IN_EMBED
 	priv->activated = TRUE;
 	if (priv->prevent_activate) return;
+#endif
 
-	g_signal_emit (toolbar, signals[NEXT], 0);
+	if (priv->typing_ahead)
+	{
+		ephy_embed_find_activate_link (get_find (toolbar), 0);
+	}
+	else
+	{
+		g_signal_emit (toolbar, signals[NEXT], 0);
+	}
 }
 
 static void
@@ -442,8 +475,10 @@ ephy_find_toolbar_init (EphyFindToolbar *toolbar)
 			  G_CALLBACK (entry_key_press_event_cb), toolbar);
 	g_signal_connect_after (priv->entry, "changed",
 				G_CALLBACK (entry_changed_cb), toolbar);
+#ifdef FIND_WHILE_TYPING_IN_EMBED
 	g_signal_connect (GTK_ENTRY (priv->entry)->im_context, "preedit-changed",
 			  G_CALLBACK (entry_preedit_changed_cb), toolbar);
+#endif
 	g_signal_connect (priv->entry, "activate",
 			  G_CALLBACK (entry_activate_cb), toolbar);
 	g_signal_connect_swapped (priv->next, "clicked",
@@ -621,10 +656,12 @@ ephy_find_toolbar_set_embed (EphyFindToolbar *toolbar,
 		g_signal_connect_object (embed, "ge-search-key-press",
 					 G_CALLBACK (tab_search_key_press_cb),
 					 toolbar, 0);
+#ifdef FIND_WHILE_TYPING_IN_EMBED
 		g_signal_connect_object (embed, "dom-mouse-click",
 					 G_CALLBACK (tab_dom_mouse_click_cb),
 					 toolbar, 0);
-#endif
+#endif /* FIND_WHILE_TYPING_IN_EMBED */
+#endif /* HAVE_TYPEAHEADFIND */
 
 		if (priv->find != NULL)
 		{
@@ -656,12 +693,14 @@ ephy_find_toolbar_find_previous (EphyFindToolbar *toolbar)
 void
 ephy_find_toolbar_open (EphyFindToolbar *toolbar,
 			gboolean links_only,
-		        gboolean clear_search)
+			gboolean typing_ahead)
 {
 	EphyFindToolbarPrivate *priv = toolbar->priv;
+	gboolean clear_search = !typing_ahead;
 
 	g_return_if_fail (priv->embed != NULL);
 
+	priv->typing_ahead = typing_ahead;
 	priv->links_only = links_only;
 	priv->explicit_focus = FALSE;
 
@@ -669,16 +708,26 @@ ephy_find_toolbar_open (EphyFindToolbar *toolbar,
 	{
 		gtk_entry_set_text (GTK_ENTRY (priv->entry), "");
 	}
+	else
+	{
+		gtk_editable_select_region (GTK_EDITABLE (priv->entry), 0, -1);
+	}
 
 	gtk_widget_show (GTK_WIDGET (toolbar));
 
-#ifdef HAVE_TYPEAHEADFIND
+#if defined(HAVE_TYPEAHEADFIND) && defined(FIND_WHILE_TYPING_IN_EMBED)
 	gtk_widget_grab_focus (GTK_WIDGET (priv->embed));
 
 	send_focus_change (priv->entry, TRUE);
+
+	if (strlen(gtk_entry_get_text (GTK_ENTRY (priv->entry))) != 0)
+	{
+		entry_changed_cb (GTK_ENTRY (priv->entry), toolbar);
+	}
 #else
 	gtk_widget_grab_focus (GTK_WIDGET (toolbar));
 #endif
+
 }
 
 void
@@ -692,7 +741,7 @@ ephy_find_toolbar_close (EphyFindToolbar *toolbar)
 
 	/* first unset explicit_focus, else we get infinite recursion */
 	priv->explicit_focus = FALSE;
-#ifndef HAVE_TYPEAHEADFIND
+#ifndef FIND_WHILE_TYPING_IN_EMBED
 	gtk_widget_grab_focus (GTK_WIDGET (priv->embed));
 #endif
 }
