@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-shell.c,v 1.129 2005/07/29 20:40:38 crispin Exp $
+ *  $Id: ephy-shell.c,v 1.130 2005/08/10 18:14:09 chpe Exp $
  */
 
 #include "config.h"
@@ -768,6 +768,8 @@ ephy_shell_new_tab_full (EphyShell *shell,
 	if (flags & EPHY_NEW_TAB_HOME_PAGE ||
 	    flags & EPHY_NEW_TAB_NEW_PAGE)
 	{
+		ephy_tab_set_typed_address (tab, "",
+					    EPHY_TAB_ADDRESS_EXPIRE_NEXT);
 		ephy_toolbar_activate_location (toolbar);
 		is_empty = load_homepage (embed);
 	}
