@@ -18,7 +18,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-tab.c,v 1.150 2005/08/01 19:28:53 chpe Exp $
+ *  $Id: ephy-tab.c,v 1.157 2005/08/20 22:22:00 chpe Exp $
  */
 
 #include "config.h"
@@ -96,15 +96,9 @@ struct _EphyTabPrivate
 	guint is_loading : 1;
 	guint is_setting_zoom : 1;
 	EphyEmbedSecurityLevel security_level;
-	/*
-	guint security_level : 3; ??
-	*/
-	/*
-	gboolean visibility;
-	gboolean load_status;
-	gboolean is_setting_zoom;
-	gboolean address_expire;
-	*/
+	/* guint security_level : 3; ? */
+	EphyTabAddressExpire address_expire;
+	/* guint address_expire : 2; ? */
 
 	/* File watch */
 	GnomeVFSMonitorHandle *monitor;
@@ -145,8 +139,6 @@ typedef struct
 static GObjectClass *parent_class = NULL;
 
 /* internal functions, accessible only from this file */
-static void	ephy_tab_set_icon_address	(EphyTab *tab,
-						 const char *location);
 static void	ephy_tab_set_load_status	(EphyTab *tab,
 						 gboolean status);
 static void	ephy_tab_set_link_message	(EphyTab *tab,
@@ -221,15 +213,18 @@ ephy_tab_set_property (GObject *object,
 
 	{
 		case PROP_TYPED_ADDRESS:
-			ephy_tab_set_typed_address (tab, g_value_get_string (value));
+			ephy_tab_set_typed_address (tab, g_value_get_string (value),
+						    EPHY_TAB_ADDRESS_EXPIRE_NOW);
 			break;
 		case PROP_POPUPS_ALLOWED:
 			ephy_tab_set_popups_allowed (tab, g_value_get_boolean (value));
 			break;
+		case PROP_ICON_ADDRESS:
+			ephy_tab_set_icon_address (tab, g_value_get_string (value));
+			break;
 		case PROP_ADDRESS:
 		case PROP_DOCUMENT_TYPE:
 		case PROP_ICON:
-		case PROP_ICON_ADDRESS:
 		case PROP_LOAD_PROGRESS:
 		case PROP_LOAD_STATUS:
 		case PROP_MESSAGE:
@@ -403,7 +398,7 @@ ephy_tab_class_init (EphyTabClass *class)
 							    G_PARAM_READABLE));
 
 	g_object_class_install_property (object_class,
-					 PROP_ICON_ADDRESS,
+					 PROP_ICON,
 					 g_param_spec_object ("icon",
 							      "Icon",
 							      "The tab icon's",
@@ -416,7 +411,7 @@ ephy_tab_class_init (EphyTabClass *class)
 							      "Icon address",
 							      "The tab icon's address",
 							      NULL,
-							      G_PARAM_READABLE));
+							      (G_PARAM_READABLE | G_PARAM_WRITABLE)));
 
 	g_object_class_install_property (object_class,
 					 PROP_LOAD_PROGRESS,
@@ -494,7 +489,7 @@ ephy_tab_class_init (EphyTabClass *class)
 							      "Typed Address",
 							      "The typed address",
 							      "",
-							      G_PARAM_READWRITE));
+							      G_PARAM_READABLE | G_PARAM_WRITABLE));
 
 	g_object_class_install_property (object_class,
 					 PROP_VISIBLE,
@@ -829,7 +824,9 @@ ephy_tab_finalize (GObject *object)
 	}
 
 	g_free (priv->title);
+	g_free (priv->loading_title);
 	g_free (priv->address);
+	g_free (priv->typed_address);
 	g_free (priv->icon_address);
 	g_free (priv->link_message);
 	g_free (priv->status_message);
@@ -903,8 +900,6 @@ ephy_tab_set_loading_title (EphyTab *tab,
 	g_free (priv->loading_title);
 	priv->loading_title = NULL;
 
-	if (title == NULL) return;
-
 	if (is_address)
 	{
 		title = freeme = get_title_from_address (title);	
@@ -929,6 +924,7 @@ ephy_tab_set_address (EphyTab *tab,
 		      char *address)
 {
 	EphyTabPrivate *priv = tab->priv;
+	GObject *object = G_OBJECT (tab);
 
 	g_free (priv->address);
 	priv->address = address;
@@ -936,14 +932,17 @@ ephy_tab_set_address (EphyTab *tab,
 	priv->is_blank = address == NULL ||
 			 strcmp (address, "about:blank") == 0;
 
-	if (priv->is_loading && priv->typed_address != NULL)
+	if (priv->is_loading &&
+	    priv->address_expire == EPHY_TAB_ADDRESS_EXPIRE_NOW &&
+	    priv->typed_address != NULL)
 	{
 		g_free (priv->typed_address);
 		priv->typed_address = NULL;
+
+		g_object_notify (object, "typed-address");
 	}
 
-	g_object_notify (G_OBJECT (tab), "address");
-	g_object_notify (G_OBJECT (tab), "typed-address");
+	g_object_notify (object, "address");
 }
 
 /* Public functions */
@@ -1202,7 +1201,7 @@ ephy_tab_icon_cache_changed_cb (EphyFaviconCache *cache,
 	}
 }
 
-static void
+void
 ephy_tab_set_icon_address (EphyTab *tab,
 			   const char *address)
 {
@@ -1418,7 +1417,7 @@ ephy_tab_open_uri_cb (EphyEmbed *embed,
 	EphyTabPrivate *priv = tab->priv;
 
 	/* Set the address here if we have a blank page.
-	 * See bug # .
+	 * See bug #147840.
 	 */
 	if (priv->is_blank)
 	{
@@ -1446,7 +1445,8 @@ ephy_tab_address_cb (EphyEmbed *embed,
 
 	/* Do not expose about:blank to the user, an empty address
 	   bar will do better */
-	if (address == NULL || strcmp (address, "about:blank") == 0)
+	if (address == NULL || address[0] == '\0' ||
+	    strcmp (address, "about:blank") == 0)
 	{
 		ephy_tab_set_address (tab, NULL);
 		ephy_tab_set_title (tab, embed, NULL);
@@ -1562,17 +1562,17 @@ ephy_tab_zoom_changed_cb (EphyEmbed *embed, float zoom, EphyTab *tab)
 }
 
 static void
-ephy_tab_title_cb (EphyEmbed *embed, EphyTab *tab)
+ephy_tab_title_cb (EphyEmbed *embed,
+		   EphyTab *tab)
 {
+	EphyTabPrivate *priv = tab->priv;
 	GObject *object = G_OBJECT (tab);
-	char *title;
-
-	title = ephy_embed_get_title (embed);
 
 	g_object_freeze_notify (object);
 
-	ephy_tab_set_title (tab, embed, title);
-	ephy_tab_set_loading_title (tab, title, FALSE);
+	ephy_tab_set_title (tab, embed, ephy_embed_get_title (embed));
+
+	ephy_tab_set_loading_title (tab, priv->title, FALSE);
 
 	g_object_thaw_notify (object);
 }
@@ -1688,7 +1688,8 @@ ensure_page_info (EphyTab *tab, EphyEmbed *embed, const char *address)
 {
 	EphyTabPrivate *priv = tab->priv;
 
-	if (priv->address == NULL || priv->address[0] != '\0')
+	if ((priv->address == NULL || priv->address[0] == '\0') &&
+	    priv->address_expire == EPHY_TAB_ADDRESS_EXPIRE_NOW)
         {
 		ephy_tab_set_address (tab, g_strdup (address));
 	}
@@ -1744,6 +1745,8 @@ ephy_tab_net_state_cb (EphyEmbed *embed,
 			g_free (priv->loading_title);
 			priv->loading_title = NULL;
 
+			priv->address_expire = EPHY_TAB_ADDRESS_EXPIRE_NOW;
+
 			g_object_notify (object, "title");
 
 			g_object_thaw_notify (object);
@@ -2008,6 +2011,7 @@ ephy_tab_init (EphyTab *tab)
 	priv->typed_address = NULL;
 	priv->title = NULL;
 	priv->loading_title = NULL;
+	priv->address_expire = EPHY_TAB_ADDRESS_EXPIRE_NOW;
 
 	embed = ephy_embed_factory_new_object (EPHY_TYPE_EMBED);
 	g_assert (embed != NULL);
@@ -2295,19 +2299,31 @@ ephy_tab_get_typed_address (EphyTab *tab)
 /**
  * ephy_tab_set_typed_address:
  * @tab: an #EphyTab
- *
+ * @address: the new typed address, or %NULL to clear it
+ * @expire: when to expire this address_expire
+ * 
  * Sets the text that @tab's #EphyWindow will display in its location toolbar
  * entry when @tab is selected.
  **/
 void
 ephy_tab_set_typed_address (EphyTab *tab,
-			    const char *address)
+			    const char *address,
+			    EphyTabAddressExpire expire)
 {
 	EphyTabPrivate *priv = tab->priv;
 
 	g_free (priv->typed_address);
-	priv->typed_address = address != NULL && address[0] != '\0'
-				? g_strdup (address) : NULL;
+	priv->typed_address = g_strdup (address);
+
+	if (expire == EPHY_TAB_ADDRESS_EXPIRE_CURRENT &&
+	    !priv->is_loading)
+	{
+		priv->address_expire = EPHY_TAB_ADDRESS_EXPIRE_NOW;
+	}
+	else
+	{
+		priv->address_expire = expire;
+	}
 
 	g_object_notify (G_OBJECT (tab), "typed-address");
 }
